<?php


/**
 * Mite - A Rapid Prototyping Micro Framework
 *
 ******************************************************************
 *    /___\        ___ _
 *    )O.o(  |\/| o | |_   BE small
 *    \_^_/  |  | | | |_   be STRONG
 *     " "
 ******************************************************************
 * @author Geoff Doty <n2geoff@gmail.com>
 * @version .76
 *
 * @copyright 2011 Geoff Doty
 * @license
 */
class Mite {

    static private $routes  = [];           // route-to-class mapping

    static private $index   = '/index.php'; //

    static private $request = NULL;         // url in address bar

    // for classes
    public $callback        = NULL;         // class/function

    static private $init    = NULL;         // ah, bad name
    static public $errors   = [];           // bites

    // unused
    static private $method  = NULL;         // for REST-ish
    static private $options = [];           // registry

    // initializes framework
    public function __construct($config = [])
    {
        $this->init($config);
    }

    public function init($name, $config = [])
    {
        // always set init (can be used as singleton later?)
        self::$init = microtime(TRUE);

        // sensible defaults
        $defaults = array
        (
            'index' => '/index.php',
            'debug' => 'FALSE',
            'paths' => array(
                'views'     => '',
                'models'    => ''
                )
        );

        // merge in configuration overriding defaults
        self::$options = array_merge($defaults, $config);

        // TODO: ack!!! forgot to update/retrieve settings from options
        self::$index = self::options('index');
    }

    /**
     * Builds route array
     *
     * Maps urls to classes or functions
     */
    public function route($url, $callback)
    {
        // for dynamic urls
        $url = preg_replace(array("@\:name@", "@\:id@"), array('([a-zA-Z\_\-]+)','([0-9]+)'), $url);

        self::$routes[$url] = $callback;
    }

    // TODO: delete/add profile (echo set options)
    public function _options()
    {
        print_r(self::$options);
    }

    //
    public function run()
    {
        $request = self::uri();

        foreach(self::$routes as $url => $callback)
        {
            preg_match("@^{$url}$@", $request, $matches);

            if(isset($matches[0]))
            {
                if(is_callable($callback))
                {
                    return call_user_func($callback);
                }
                else
                {
                    // $class = get_class($this); // 5 +
                    // $class = get_called_class(); // 5.3

                    // if(is_callable("{$class}::{$callback}"))
                    if(is_callable([$this, $callback]))
                    {
                        // return call_user_func("{$class}::{$callback}");  // called staticly 5.2.3+
                        // return call_user_func(array($class, $callback)); // called?
                        return $this->$callback();
                    }
                    else
                    {
                        echo 'PROBLEM';
                    }
                    // header("HTTP/1.1 404 Not Found");
                    exit('Function Not Found');
                }
            }
        }

        // header("HTTP/1.1 404 Not Found");
        exit('Page Not Found');

    }

    //TODO:  index.php required in url should make it either or?  hmm htaccess issue
    public function uri($raw = FALSE)
    {
        // cleans and strips query string
        // $uri = str_replace($this->controller, '', preg_replace('/\?.+/', '', $_SERVER['REQUEST_URI']));
        $uri = str_replace(self::$index, '', preg_replace('/\?.+/', '', $_SERVER['REQUEST_URI']));

        return ($uri == '' OR $uri == '//') ? '/' : $uri;
    }

    function options($key = NULL, $value = NULL)
    {
        if($key === NULL)
        {
            return self::$options;
        }
        else
        {
            if($value === NULL)
            {
                return isset(self::$options[$key]) ? self::$options[$key] : NULL;
            }
            else
            {
                return isset(self::$options[$key][$value]) ? self::$options[$key][$value] : NULL;
            }
        }
    }

    /*****************************************************************
     * State Management
     *****************************************************************/
    /**
     * Cookie Management
     *
     * set/get cookies by call signature
     */
    public function cookie($name, $value = NULL, $expire = 0, $path = '', $domain = '', $secure = FALSE, $httponly = false)
    {
        if($value === NULL)
        {   // read cookie
            return isset($_COOKIE[$name]) ? $_COOKIE[$name] : NULL ;
        }
        else
        {   // create cookie
            setcookie($name, $value, $expire, $path, $domain, $secure, $httponly);
        }
    }

    /**
     * Flash Notice
     *
     * Creates cookie expires on next request
     */
    public function flash($name, $value)
    {
        self::cookie($name, $value, time()+1);
    }

    public function session($key, $value = NULL)
    {
        if($value === NULL)
        {   // read session
            return isset($_SESSION[$key]) ? $_SESSION[$key] : NULL ;
        }
        else
        {  // create session
           $_SESSION[$key] = $value;
        }
    }

    /*****************************************************************
     * Helper Methods
     *****************************************************************/

    /**
     * Return Uri Segment
     */
    public function uri_segment($number)
    {
        $segments = explode('/', trim(self::$request, '/'));

        return isset($segments[($number - 1)]) ? $segments[($number - 1)] : NULL;
    }

    public function input($key, $clean = TRUE)
    {   // merge global input arrays
        $input = array_merge($_GET, $_POST);

        if($clean === TRUE)
        {
            return isset($input[$key]) ? self::xss($input[$key]) : NULL;
        }
        else
        {
            return isset($input[$key]) ? $input[$key] : NULL;
        }
    }

    public function xss($value)
    {
        return htmlentities(stripslashes($value));
    }

    public function style($file = NULL)
    {
        if($file === NULL)
        {
            $style  = "<style type='text/css'>\n";
            $style .= "\t html,body,div,p{margin:0;padding:0;border:0;}";
            // $style .= ".main{margin: 0 auto; width: 600px;}";
            $style .= ".profile{background-color: #e3e3e3;font-weight: bold; border-bottom: 1px solid #404040;}";
            $style .= ".dp20,.dp25,.dp33,.dp50,.dp100{float:left;display:inline;*margin-left:-0.04em;}";
            $style .= ".dp20{width:20%;}.dp25{width:25%;}.dp33{width:33.33%;}.dp50{width:50%;}.dp100{width:100%;}";
            $style .= ".clear{clear:both;}";
            $style .= "\n</style>\n";
        }

        return $style;
    }

    public function view($template, $data = NULL)
    {
        if($data != NULL)
        {
            if(is_array($data))
            {
                extract($data);
            }
        }

        // check if folder defined
        $dir = self::options('paths','views');

        // update path
        $path = $dir ? "{$dir}/{$template}" : "{$template}";

        include($path);
    }

    /*****************************************************************
     * Error Handling and Logging
     *****************************************************************/

    public function errors() {}
    protected function has_error()
    {
        if(count(self::$errors) > 0)
        {
            return TRUE;
        }

        return FALSE;
    }
    public function log($message) {}

    /*****************************************************************
     * Profiling
     *****************************************************************/

    function explain()
    {
        $output = "\n<!-- Debugging Information -->\n";
        $output .= Mite::style();

        $output .= "<div class='main'>\n";
        $output .= "<fieldset>\n\t<legend>Debugging Information</legend>\n";

        // Routing
        $output .= "<div class='dp100 profile'>\n";
        $output .= "\t<div class='dp33'><b>PAGE REQUEST</b></div>\n";
        $output .= "\t<div class='dp33'>" . self::uri() . "</div>\n";
        $output .= "\t<div class='dp33'>&nbsp;</div>\n";
        $output .= "</div>\n";

        $output .= "<div class='dp100'>\n";
        foreach(self::$routes as $route => $callback)
        {
            $output .= "\t<div class='dp33'>&nbsp;</div>\n"; // future request method
            $output .= "\t<div class='dp33'>{$route}</div>\n";
            $output .= "\t<div class='dp33'>";
            $output .= is_object($callback) ? get_class($callback) : $callback;
            $output .= "</div>\n";
        }
        $output .= "</div>\n";

        // Request variables
        $output .= "<div class='dp100 profile'>GET DATA</div>\n";
        foreach($_GET as $key => $value)
        {
            $output .= "\t<div class='dp50'>{$key}</div>\n";
            $output .= "\t<div class='dp50'>{$value}</div>\n";
        }

        $output .= "<div class='dp100 profile'>POST DATA</div>\n";
        foreach($_POST as $key => $value)
        {
            $output .= "\t<div class='dp50'>{$key}</div>\n";
            $output .= "\t<div class='dp50'>{$value}</div>\n";
        }

        $output .= "<div class='dp100 profile'>COOKIE DATA</div>\n";
        foreach($_COOKIE as $key => $value)
        {
            $output .= "\t<div class='dp50'>{$key}</div>\n";
            $output .= "\t<div class='dp50'>{$value}</div>\n";
        }

        $output .= "<div class='dp100 profile'>SESSION DATA</div>\n";
        if(isset($_SESSION))
        {
            foreach((array) $_SESSION as $key => $value)
            {
                $output .= "\t<div class='dp50'>{$key}</div>\n";
                $output .= "\t<div class='dp50'>{$value}</div>\n";
            }
        }

        // continued in __destruct()
        return $output;
    }

    public function __destruct()
    {
        if(self::options('debug') === TRUE)
        {
            $output = self::explain();

            // Memory
            $output .= "<div class='dp100 profile'>MEMORY USAGE</div>\n";
            $output .= "\t<div class='dp33'>&nbsp;</div>\n";
            $output .= "\t<div class='dp33'>" .  round(memory_get_usage()/1024,2) . " KBs</div>\n";
            $output .= "\t<div class='dp33'>" .  round(memory_get_peak_usage()/1024,2) . " KBs</div>\n";
            // $output .= "</div>\n";

            // Profiling
            $output .= "<div class='dp100 profile'>PROFILING USAGE</div>\n";
            $output .= "\t<div class='dp50'>&nbsp;</div>\n";
            $output .= "\t<div class='dp50'>" . round(microtime(true) - self::$init, 4) . " Seconds</div>\n";
            // $output .= "</div>\n";

            $output .= "</fieldset>\n</div>\n";
            $output .= "\n<!-- End of Debugging Information -->\n";

            echo $output;
        }
    }
}

// Basic CRUD class, may remove, or make more query builder style
abstract class Mite_Model {

    protected $_table; // table name
    protected $_key; // primary key
    private $db;

    public function __construct($dsn, $username = NULL, $password = NULL)
    {
        $this->db = $this->connect($dsn, $username, $password);
    }

    public function connect($dsn, $username = NULL, $password = NULL)
    {
        // $dsn = "{$driver}:dbname={$database};host={$host}";

        try
        {
            return new PDO($dsn, $username, $password);
        }
        catch (PDOException $e)
        {
            throw new Exception($e->getMessage());
        }
    }

    // basic crud
    public function insert($data)
    {
       $sql = "INSERT INTO {$this->_table} ";

       $columns = array();
       $values  = array();

       foreach($data as $column => $value)
       {
           $columns[] = $column;
           $values[]  = $value;
       }

       $sql .= '(' . implode(',', $columns) . ') ';
       $sql .= 'VALUES(' . implode(',', $values) . ')';

       return $this->db->exec($sql);
    }

    public function update($data)
    {
       $sql = "UPDATE {$this->_table} SET ";

       $updates = array();
       foreach($data as $column => $value)
       {
           $updates[] = "{$column}='{$value}'";
       }

       $sql .= implode(',', $updates);
       $sql .= "WHERE {$this->_key}='{$value[$this->_key]}'";

       return $this->db->exec($sql);
    }

    public function delete($data)
    {
        $sql = "DELETE FROM {$this->_table} ";

        $deletes = array();
        foreach($data as $column => $value)
        {
           $deletes[] = "{$column}='{$value}'";
        }

        $sql .= implode(',', $deletes);
        $sql .= "WHERE {$column}='{$value}'";

        return $this->db->exec($sql);
    }

    public function find($value)
    {
        $sql = "SELECT * FROM {$this->_table} WHERE {$this->_key} = '{$value}'";

        return $this->db->query($sql);
    }

    // raw sql
    public function query($sql)
    {
        return $this->db->query($sql);
    }

    public function last_id()
    {
        try
        {
            return $this->db->lastInsertId();
        }
        catch(PDOException $e)
        {
            return NULL;
        }
    }
}
